use crate::vm::object::DvmObject;
use std::rc::Rc;

#[derive(Debug, Clone, PartialEq)]
pub enum DvmValue {
    Byte(u8),
    Int(i32),
    Long(i64),
    Double(f64),
    Boolean(bool),
    Object(Rc<DvmObject>),
    USize(usize),
    Uninit,
    Void,
}

impl DvmValue {
    pub fn expect_long(&self) -> i64 {
        if let DvmValue::Long(l) = self {
            *l
        } else {
            panic!("Expected DvmValue::Long, but found DvmValue::{:?}", self)
        }
    }

    pub fn expect_object(&self) -> Rc<DvmObject> {
        if let DvmValue::Object(o) = self {
            o.clone()
        } else {
            panic!("Expected DvmValue::Object, but found DvmValue::{:?}", self);
        }
    }
    
    pub fn is_object(&self) -> bool {
        match self {
            DvmValue::Object(_) => true,
            _ => false,
        }
    }

    pub fn expect_usize(&self) -> usize {
        if let DvmValue::USize(u) = self {
            *u
        } else {
            panic!("Expected DvmValue::USize, but found DvmValue::{:?}", self);
        }
    }
}
